@@ -1,7 +1,7 @@
 #
 # Chris Lumens <clumens@redhat.com>
 #
-# Copyright 2006, 2007, 2008, 2009, 2010 Red Hat, Inc.
+# Copyright 2006-2015 Red Hat, Inc.
 #
 # This copyrighted material is made available to anyone wishing to use, modify,
 # copy, or redistribute it subject to the terms and conditions of the GNU
@@ -15,7 +15,7 @@
 # Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
 # trademarks that are incorporated in the source code or documentation are not
 # subject to the GNU General Public License and may only be used or replicated
-# with the express permission of Red Hat, Inc. 
+# with the express permission of Red Hat, Inc.
 #
 """
 Methods for working with kickstart versions.
@@ -39,17 +39,20 @@ This module also exports several functions:
 
     versionToString - Perform the reverse mapping.
 
+    versionToLongString - Perform the reverse mapping but use long names.
+
     versionFromFile - Read a kickstart file and determine the version of
                       syntax it uses.  This requires the kickstart file to
                       have a version= comment in it.
 """
-import imputil, re, sys
-from urlgrabber import urlopen
+import os, re, sys
 
-import gettext
-_ = lambda x: gettext.ldgettext("pykickstart", x)
+import importlib
+
+from pykickstart.i18n import _
 
 from pykickstart.errors import KickstartVersionError
+from pykickstart.load import load_to_str
 
 # Symbolic names for internal version numbers.
 RHEL3 = 900
@@ -59,7 +62,7 @@ FC4 = 2000
 FC5 = 3000
 FC6 = 4000
 RHEL5 = 4100
-F7  = 5000
+F7 = 5000
 F8 = 6000
 F9 = 7000
 F10 = 8000
@@ -70,17 +73,37 @@ RHEL6 = 11100
 F14 = 12000
 F15 = 13000
 F16 = 14000
+F17 = 15000
+F18 = 16000
+F19 = 17000
+F20 = 18000
+F21 = 19000
+RHEL7 = 19100
+F22 = 20000
+F23 = 21000
+F24 = 22000
+F25 = 23000
+F26 = 24000
+F27 = 25000
+F28 = 26000
+F29 = 27000
+F30 = 30000
+RHEL8 = 26100
 
 # This always points at the latest version and is the default.
-DEVEL = F16
+DEVEL = F30
 
 # A one-to-one mapping from string representations to version numbers.
 versionMap = {
-        "DEVEL": DEVEL,
-        "FC3": FC3, "FC4": FC4, "FC5": FC5, "FC6": FC6, "F7": F7, "F8": F8,
-        "F9": F9, "F10": F10, "F11": F11, "F12": F12, "F13": F13,
-        "F14": F14, "F15": F15, "F16": F16,
-        "RHEL3": RHEL3, "RHEL4": RHEL4, "RHEL5": RHEL5, "RHEL6": RHEL6
+    "DEVEL": DEVEL,
+    "FC3": FC3, "FC4": FC4, "FC5": FC5, "FC6": FC6, "F7": F7, "F8": F8,
+    "F9": F9, "F10": F10, "F11": F11, "F12": F12, "F13": F13,
+    "F14": F14, "F15": F15, "F16": F16, "F17": F17, "F18": F18,
+    "F19": F19, "F20": F20, "F21": F21, "F22": F22, "F23": F23,
+    "F24": F24, "F25": F25, "F26": F26, "F27": F27, "F28": F28,
+    "F29": F29, "F30": F30,
+    "RHEL3": RHEL3, "RHEL4": RHEL4, "RHEL5": RHEL5, "RHEL6": RHEL6,
+    "RHEL7": RHEL7, "RHEL8": RHEL8
 }
 
 def stringToVersion(s):
@@ -94,21 +117,21 @@ def stringToVersion(s):
         pass
 
     # Now try the Fedora versions.
-    m = re.match("^fedora.* (\d+)$", s, re.I)
+    m = re.match(r"^fedora.* (\d+)$", s, re.I)      # pylint: disable=no-member
 
     if m and m.group(1):
-        if versionMap.has_key("FC" + m.group(1)):
+        if "FC" + m.group(1) in versionMap:
             return versionMap["FC" + m.group(1)]
-        elif versionMap.has_key("F" + m.group(1)):
+        elif "F" + m.group(1) in versionMap:
             return versionMap["F" + m.group(1)]
         else:
             raise KickstartVersionError(_("Unsupported version specified: %s") % s)
 
     # Now try the RHEL versions.
-    m = re.match("^red hat enterprise linux.* (\d+)([\.\d]*)$", s, re.I)
+    m = re.match(r"^red hat enterprise linux.* (\d+)([\.\d]*)$", s, re.I)       # pylint: disable=no-member
 
     if m and m.group(1):
-        if versionMap.has_key("RHEL" + m.group(1)):
+        if "RHEL" + m.group(1) in versionMap:
             return versionMap["RHEL" + m.group(1)]
         else:
             raise KickstartVersionError(_("Unsupported version specified: %s") % s)
@@ -124,7 +147,7 @@ def versionToString(version, skipDevel=False):
     if not skipDevel and version == versionMap["DEVEL"]:
         return "DEVEL"
 
-    for (key, val) in versionMap.iteritems():
+    for (key, val) in list(versionMap.items()):
         if key == "DEVEL":
             continue
         elif val == version:
@@ -132,37 +155,36 @@ def versionToString(version, skipDevel=False):
 
     raise KickstartVersionError(_("Unsupported version specified: %s") % version)
 
+def versionToLongString(version):
+    """
+        Convert version into a long string representation.
+    """
+    result = versionToString(version, True)
+    result = result.replace('FC', 'F').replace('F', 'Fedora')
+    result = result.replace('RHEL', 'RedHatEnterpriseLinux')
+    return result
+
 def versionFromFile(f):
     """Given a file or URL, look for a line starting with #version= and
        return the version number.  If no version is found, return DEVEL.
     """
     v = DEVEL
 
-    fh = urlopen(f)
+    contents = load_to_str(f)
 
-    while True:
-        try:
-            l = fh.readline()
-        except StopIteration:
-            break
-
-        # At the end of the file?
-        if l == "":
-            break
-
-        if l.isspace() or l.strip() == "":
+    for line in contents.splitlines(True):
+        if line.strip() == "":
             continue
 
-        if l[:9] == "#version=":
-            v = stringToVersion(l[9:].rstrip())
+        if line[:9] == "#version=":
+            v = stringToVersion(line[9:].rstrip())
             break
 
-    fh.close()
     return v
 
 def returnClassForVersion(version=DEVEL):
     """Return the class of the syntax handler for version.  version can be
-       either a string or the matching constant.  Raises KickstartValueError
+       either a string or the matching constant.  Raises KickstartVersionError
        if version does not match anything.
     """
     try:
@@ -175,12 +197,11 @@ def returnClassForVersion(version=DEVEL):
     module = module.lower()
 
     try:
-        import pykickstart.handlers
-        sys.path.extend(pykickstart.handlers.__path__)
-        found = imputil.imp.find_module(module)
-        loaded = imputil.imp.load_module(module, found[0], found[1], found[2])
+        _path = os.path.join(os.path.dirname(__file__), "handlers/")
+        sys.path.extend([_path])
+        loaded = importlib.import_module(module)
 
-        for (k, v) in loaded.__dict__.iteritems():
+        for (k, v) in list(loaded.__dict__.items()):
             if k.lower().endswith("%shandler" % module):
                 return v
     except:
@@ -195,3 +216,6 @@ def makeVersion(version=DEVEL):
     """
     cl = returnClassForVersion(version)
     return cl()
+
+def getVersionFromCommandClass(cls):
+    return versionMap[cls.__name__.split('_')[0]]
