@@ -1,13 +1,13 @@
-import unittest, shlex
-import warnings
+import six
+import sys
+import unittest
 import tempfile
-from tests.baseclass import *
+import os
 
-from pykickstart.version import *
-from pykickstart.errors import *
-from pykickstart.commands.vnc import *
-#from pykickstart.base import *
-#from pykickstart.options import *
+from tests.baseclass import CommandTest, loadModules
+
+from pykickstart.version import *           # pylint: disable=wildcard-import
+from pykickstart.errors import KickstartVersionError
 
 def getClassName(cls):
     return cls().__class__.__name__
@@ -35,6 +35,9 @@ class StringToVersion_TestCase(CommandTest):
         self.assertRaises(KickstartVersionError, stringToVersion, "F1111")
         self.assertRaises(KickstartVersionError, stringToVersion, "F 11")
 
+        # fail - doesn't exist
+        self.assertRaises(KickstartVersionError, stringToVersion, "Red hat Enterprise Linux 1")
+
         # pass - FC3
         self.assertEqual(stringToVersion("Fedora Core 3"), FC3)
         self.assertEqual(stringToVersion("FC3"), FC3)
@@ -81,6 +84,45 @@ class StringToVersion_TestCase(CommandTest):
         # pass - F16
         self.assertEqual(stringToVersion("Fedora 16"), F16)
         self.assertEqual(stringToVersion("F16"), F16)
+        # pass - F17
+        self.assertEqual(stringToVersion("Fedora 17"), F17)
+        self.assertEqual(stringToVersion("F17"), F17)
+        # pass - F18
+        self.assertEqual(stringToVersion("Fedora 18"), F18)
+        self.assertEqual(stringToVersion("F18"), F18)
+        # pass - F19
+        self.assertEqual(stringToVersion("Fedora 19"), F19)
+        self.assertEqual(stringToVersion("F19"), F19)
+        # pass - F20
+        self.assertEqual(stringToVersion("Fedora 20"), F20)
+        self.assertEqual(stringToVersion("F20"), F20)
+        # pass - F21
+        self.assertEqual(stringToVersion("Fedora 21"), F21)
+        self.assertEqual(stringToVersion("F21"), F21)
+        # pass - F22
+        self.assertEqual(stringToVersion("Fedora 22"), F22)
+        self.assertEqual(stringToVersion("F22"), F22)
+        # pass - F23
+        self.assertEqual(stringToVersion("Fedora 23"), F23)
+        self.assertEqual(stringToVersion("F23"), F23)
+        # pass - F24
+        self.assertEqual(stringToVersion("Fedora 24"), F24)
+        self.assertEqual(stringToVersion("F24"), F24)
+        # pass - F25
+        self.assertEqual(stringToVersion("Fedora 25"), F25)
+        self.assertEqual(stringToVersion("F25"), F25)
+        # pass - F26
+        self.assertEqual(stringToVersion("Fedora 26"), F26)
+        self.assertEqual(stringToVersion("F26"), F26)
+        # pass - F27
+        self.assertEqual(stringToVersion("Fedora 27"), F27)
+        self.assertEqual(stringToVersion("F27"), F27)
+        # pass - F28
+        self.assertEqual(stringToVersion("Fedora 28"), F28)
+        self.assertEqual(stringToVersion("F28"), F28)
+        # pass - F29
+        self.assertEqual(stringToVersion("Fedora 29"), F29)
+        self.assertEqual(stringToVersion("F29"), F29)
 
         # pass - RHEL3
         self.assertEqual(stringToVersion("Red Hat Enterprise Linux 3"), RHEL3)
@@ -102,7 +144,7 @@ class StringToVersion_TestCase(CommandTest):
         self.assertEqual(stringToVersion("Red Hat Enterprise Linux 5"), RHEL5)
         self.assertEqual(stringToVersion("Red Hat Enterprise Linux Client 5"), RHEL5)
         self.assertEqual(stringToVersion("Red Hat Enterprise Linux Server 5"), RHEL5)
-        for MINOR in range(1,10):
+        for MINOR in range(1, 10):
             self.assertEqual(stringToVersion("Red Hat Enterprise Linux 5.%s" % MINOR), RHEL5)
             self.assertEqual(stringToVersion("Red Hat Enterprise Linux Client 5.%s" % MINOR), RHEL5)
             self.assertEqual(stringToVersion("Red Hat Enterprise Linux Server 5.%s" % MINOR), RHEL5)
@@ -112,19 +154,35 @@ class StringToVersion_TestCase(CommandTest):
         self.assertEqual(stringToVersion("Red Hat Enterprise Linux 6"), RHEL6)
         self.assertEqual(stringToVersion("Red Hat Enterprise Linux Client 6"), RHEL6)
         self.assertEqual(stringToVersion("Red Hat Enterprise Linux Server 6"), RHEL6)
-        for MINOR in range(1,10):
+        for MINOR in range(1, 10):
             self.assertEqual(stringToVersion("Red Hat Enterprise Linux 6.%s" % MINOR), RHEL6)
             self.assertEqual(stringToVersion("Red Hat Enterprise Linux Client 6.%s" % MINOR), RHEL6)
             self.assertEqual(stringToVersion("Red Hat Enterprise Linux Server 6.%s" % MINOR), RHEL6)
         self.assertEqual(stringToVersion("RHEL6"), RHEL6)
 
+        # pass - RHEL7
+        self.assertEqual(stringToVersion("Red Hat Enterprise Linux 7"), RHEL7)
+        self.assertEqual(stringToVersion("Red Hat Enterprise Linux Client 7"), RHEL7)
+        self.assertEqual(stringToVersion("Red Hat Enterprise Linux Server 7"), RHEL7)
+        for MINOR in range(1, 10):
+            self.assertEqual(stringToVersion("Red Hat Enterprise Linux 7.%s" % MINOR), RHEL7)
+            self.assertEqual(stringToVersion("Red Hat Enterprise Linux Client 7.%s" % MINOR), RHEL7)
+            self.assertEqual(stringToVersion("Red Hat Enterprise Linux Server 7.%s" % MINOR), RHEL7)
+        self.assertEqual(stringToVersion("RHEL7"), RHEL7)
+
+        # pass - RHEL8
+        self.assertEqual(stringToVersion("Red Hat Enterprise Linux 8"), RHEL8)
+        for MINOR in range(1,10):
+            self.assertEqual(stringToVersion("Red Hat Enterprise Linux 8.%s" % MINOR), RHEL8)
+        self.assertEqual(stringToVersion("RHEL8"), RHEL8)
+
 class VersionToString_TestCase(CommandTest):
     def runTest(self):
 
         # Make sure DEVEL is the highest version, but RHEL versions aren't
         # counted as devel.
         highest = 0
-        for (ver_str,ver_num) in versionMap.items():
+        for (ver_str, ver_num) in list(versionMap.items()):
             if ver_str.startswith("RHEL"):
                 continue
 
@@ -150,31 +208,62 @@ class VersionToString_TestCase(CommandTest):
         self.assertEqual(versionToString(F14, skipDevel=True), "F14")
         self.assertEqual(versionToString(F15, skipDevel=True), "F15")
         self.assertEqual(versionToString(F16, skipDevel=True), "F16")
-        self.assertEqual(versionToString(F16, skipDevel=False), "DEVEL")
+        self.assertEqual(versionToString(F17, skipDevel=True), "F17")
+        self.assertEqual(versionToString(F18, skipDevel=True), "F18")
+        self.assertEqual(versionToString(F19, skipDevel=True), "F19")
+        self.assertEqual(versionToString(F20, skipDevel=True), "F20")
+        self.assertEqual(versionToString(F21, skipDevel=True), "F21")
+        self.assertEqual(versionToString(F22, skipDevel=True), "F22")
+        self.assertEqual(versionToString(F23, skipDevel=True), "F23")
+        self.assertEqual(versionToString(F24, skipDevel=True), "F24")
+        self.assertEqual(versionToString(F25, skipDevel=True), "F25")
+        self.assertEqual(versionToString(F26, skipDevel=True), "F26")
+        self.assertEqual(versionToString(F27, skipDevel=True), "F27")
+        self.assertEqual(versionToString(F28, skipDevel=True), "F28")
+        self.assertEqual(versionToString(F29, skipDevel=True), "F29")
+        self.assertEqual(versionToString(F30, skipDevel=True), "F30")
+        self.assertEqual(versionToString(F30, skipDevel=False), "DEVEL")
         # RHEL series
         self.assertEqual(versionToString(RHEL3), "RHEL3")
         self.assertEqual(versionToString(RHEL4), "RHEL4")
         self.assertEqual(versionToString(RHEL5), "RHEL5")
         self.assertEqual(versionToString(RHEL6), "RHEL6")
+        self.assertEqual(versionToString(RHEL7), "RHEL7")
+        self.assertEqual(versionToString(RHEL8), "RHEL8")
+
+        # fail
+        self.assertRaises(KickstartVersionError, versionToString, 47)
 
 class returnClassForVersion_TestCase(CommandTest):
     def runTest(self):
 
         # Test that everything in version.versionMap has a handler, except
         # for DEVEL.
-        for (name, vers) in versionMap.iteritems():
+        for (name, vers) in list(versionMap.items()):
             if name == "DEVEL":
                 continue
 
             self.assertEqual(returnClassForVersion(vers).version, vers)
 
+        # test that unknown version will raise an exception
+        import pykickstart.version as ver
+        orig_versionToString = ver.versionToString
+        with self.assertRaises(KickstartVersionError):
+            def fake_versionToString(version, skipDevel=False):
+                return "-1"
+
+            ver.versionToString = fake_versionToString
+            ver.returnClassForVersion(-1)
+        ver.versionToString = orig_versionToString
+
         # Load the handlers
-        import pykickstart.handlers
-        for module in loadModules(pykickstart.handlers.__path__[0], cls_pattern="Handler", skip_list=["control"]):
+        _path = os.path.join(os.path.dirname(__file__), "..", "pykickstart", "handlers")
+        _path = os.path.abspath(_path)
+        for module in loadModules(_path, cls_pattern="Handler", skip_list=["control"]):
             if module.__name__.endswith("Handler") and module.__name__ not in ["BaseHandler"]:
                 # What is the version of the handler?
-                vers = module.__name__.replace("Handler","")
-                self.assertTrue(versionMap.has_key(vers))
+                vers = module.__name__.replace("Handler", "")
+                self.assertTrue(vers in versionMap)
                 # Ensure that returnClassForVersion returns what we expect
                 self.assertEqual(getClassName(returnClassForVersion(versionMap[vers])), getClassName(module))
 
@@ -182,7 +271,9 @@ class versionFromFile_TestCase(CommandTest):
     def runTest(self):
 
         def write_ks_cfg(buf):
-            (fd, name) = tempfile.mkstemp(prefix="ks-", suffix=".cfg", dir="/tmp")
+            (fd, name) = tempfile.mkstemp(prefix="ks-", suffix=".cfg", dir="/tmp", text=True)
+            if six.PY3:
+                buf = buf.encode(sys.getdefaultencoding())
             os.write(fd, buf)
             os.close(fd)
             return name
@@ -241,5 +332,25 @@ cdrom
         self.assertRaises(KickstartVersionError, versionFromFile, ks_cfg)
         os.unlink(ks_cfg)
 
+class FailedImpImport_TestCase(CommandTest):
+    def runTest(self):
+        try:
+            # will force another import
+            del sys.modules['pykickstart.version']
+            del sys.modules['importlib']
+
+            import pykickstart.version as ver
+            from pykickstart.handlers.f23 import F23Handler
+
+            cls = ver.returnClassForVersion(ver.F23)
+
+            # assert the names; b/c of how the importlib.import_module() works
+            # asserting both classes being equal fails
+            self.assertEqual(cls.__name__, F23Handler.__name__)
+        finally:
+            # force import to reload these modules
+            del sys.modules['pykickstart.version']
+            del sys.modules['importlib']
+
 if __name__ == "__main__":
     unittest.main()
